XVECTOR2 = { XType = 'XVECTOR2' }
setmetatable(XVECTOR2, XVECTOR2)

function XVECTOR2:Normalize()
    local mag = self:Magnitude()
    if mag < 1e-6 and mag > -1e-6 then
        self:Clear()
        mag = 0
    else
        local f = 1.0 / mag
        self.x = self.x * f
        self.y = self.y * f
    end
    return mag
end
function XVECTOR2:Set(x, y)
    self.x = x
    self.y = y
end
function XVECTOR2:Clear()
    self.x = 0
    self.y = 0
end
function XVECTOR2:IsZero()
    return self.x == 0 and self.y == 0
end
function XVECTOR2:IsEquals(v, e)
    return X_Equals(self.x, v.x, e) and X_Equals(self.y, v.y, e)
end
function XVECTOR2:Magnitude()
    return math.sqrt(self.x ^ 2 + self.y ^ 2)
end
function XVECTOR2:SquaredMagnitude()
    return self.x ^ 2 + self.z ^ 2
end
function XVECTOR2:MinMemberValue()
    return math.min(self.x, self.y)
end
function XVECTOR2:MaxMemberValue()
    return math.max(self.x, self.y)
end
function XVECTOR2:Snap()
    local m = self.m
    for i = 1, 2 do
        if m[i] > 1.0 - 1e-5 then
            self:Clear()
            m = self.m
            m[i] = 1
            break
        elseif m[i] < -1.0 + 1e-5 then
            self:Clear()
            m = self.m
            m[i] = -1
            break
        end
    end
    self:Set(m[1], m[2], m[3])
end
function XVECTOR2:AddVecWith(v)
    self.x = self.x + v.x
    self.y = self.y + v.y
end
function XVECTOR2:SubVecWith(v)
    self.x = self.x - v.x
    self.y = self.y - v.y
end
function XVECTOR2:MulFloatWith(v)
    self.x = self.x * v
    self.y = self.y * v
end
function XVECTOR2:DevFloatWith(v)
    self.x = self.x / v
    self.y = self.y / v
end

function XVECTOR2.__add(v1, v2)
    if math.isVec2(v1) and math.isVec2(v2) then
        return XVECTOR2(v1.x + v2.x, v1.y + v2.y)
    end
    error('Illegal operation: ' .. math.typeName(v1) .. ' + ' .. math.typeName(v2))
end
function XVECTOR2.__sub(v1, v2)
    if math.isVec2(v1) and math.isVec2(v2) then
        return XVECTOR2(v1.x - v2.x, v1.y - v2.y)
    end
    error('Illegal operation: ' .. math.typeName(v1) .. ' - ' .. math.typeName(v2))
end
function XVECTOR2.__mul(arg1, arg2)
    if math.isVec2(arg2) and type(arg1) == 'number' then
        --- float * vec2
        return XVECTOR2(arg1 * arg2.x, arg1 * arg2.y)
    elseif math.isVec2(arg1) and type(arg2) == 'number' then
        --- vec2 * float
        return XVECTOR2(arg1.x * arg2, arg1.y * arg2)
    elseif math.isVec2(arg1) and math.isVec2(arg2) then
        --- vec2 *vec2
        return XVECTOR2(arg1.x * arg2.x, arg1.y * arg2.y)
    end
    error('Illegal operation: ' .. math.typeName(arg1) .. ' * ' .. math.typeName(arg2))
end
function XVECTOR2.__div(arg1, arg2)
    if math.isVec2(arg1) and type(arg2) == 'number' then
        --- vec2 / number
        return XVECTOR2(arg1.x / arg2, arg1.y / arg2)
    elseif math.isVec2(arg1) and math.isVec2(arg2) then
        --- vec2 / vec2
        return XVECTOR2(arg1.x / arg2.x, arg1.y / arg2.y)
    end
    error('Illegal operation: ' .. math.typeName(arg1) .. ' / ' .. math.typeName(arg2))
end
function XVECTOR2.__eq(v1, v2)
    if math.isVec2(v1) and math.isVec2(v2) then
        return v1.x == v2.x and v1.y == v2.y
    end
    return false
end
function XVECTOR2:__tostring()
    return 'XVECTOR2(x=' .. self.x .. ', y=' .. self.y .. ')'
end
function XVECTOR2:__index(key)
    if key == 'm' then
        return { self.x, self.y }
    end
    return XVECTOR2[key]
end
function XVECTOR2:__call(...)
    local vec2 = {}
    setmetatable(vec2, XVECTOR2)
    local args = { ... }

    if #args == 0 then
        vec2.x = 0
        vec2.y = 0
    elseif #args == 1 then
        if math.isNumber(args[1]) then
            vec2.x = args[1]
            vec2.y = args[1]
        elseif math.isVec2(args[1]) then
            vec2.x = args[1].x
            vec2.y = args[1].y
        else
            error("XVECTOR2 构造参数错误")
        end
    elseif #args == 2 then
        vec2.x = args[1]
        vec2.y = args[2]
    else
        error("XVECTOR2 构造参数错误")
    end

    return vec2
end
