XVECTOR3 = { XType = 'XVECTOR3' }
setmetatable(XVECTOR3, XVECTOR3)

function XVECTOR3:Set(x, y, z)
    self.x = x
    self.y = y
    self.z = z
end
function XVECTOR3:Clear()
    self.x = 0
    self.y = 0
    self.z = 0
end
function XVECTOR3:IsZero()
    return self.x == 0 and self.y == 0 and self.z == 0
end
function XVECTOR3:IsEquals(v, e)
    return X_Equals(self.x, v.x, e) and X_Equals(self.y, v.y, e) and X_Equals(self.z, v.z, e)
end
function XVECTOR3:Magnitude()
    return math.sqrt(self.x ^ 2 + self.y ^ 2 + self.z ^ 2)
end
function XVECTOR3:MagnitudeH()
    return math.sqrt(self.x ^ 2 + self.z ^ 2)

end
function XVECTOR3:SquaredMagnitude()
    return self.x ^ 2 + self.y ^ 2 + self.z ^ 2
end
function XVECTOR3:SquaredMagnitudeH()
    return self.x ^ 2 + self.z ^ 2
end
function XVECTOR3:MinMember()
    return math.min(self.x, self.y, self.z)
end
function XVECTOR3:MaxMember()
    return math.max(self.x, self.y, self.z)
end
function XVECTOR3:Snap()
    local m = self.m
    for i = 1, 3 do
        if m[i] > 1.0 - 1e-5 then
            self:Clear()
            m = self.m
            m[i] = 1
            break
        elseif m[i] < -1.0 + 1e-5 then
            self:Clear()
            m = self.m
            m[i] = -1
            break
        end
    end
    self:Set(m[1], m[2], m[3])
end
function XVECTOR3:Normalize()
    local mag = self:Magnitude()
    if mag < 1e-6 and mag > -1e-6 then
        self:Clear()
        mag = 0
    else
        local f = 1 / mag
        self.x = self.x * f
        self.y = self.y * f
        self.z = self.z * f
    end
    return mag
end
function XVECTOR3:CrossProduct(arg1, arg2)
    self.x = arg1.y * arg2.z - arg1.z * arg2.y
    self.y = arg1.z * arg2.x - arg1.x * arg2.z
    self.z = arg1.x * arg2.y - arg1.y * arg2.x
end
function XVECTOR3:AddVecWith(vec2)
    self.x = self.x + vec2.x
    self.y = self.y + vec2.y
    self.z = self.z + vec2.z
    return self
end
function XVECTOR3:SubVecWith(vec2)
    self.x = self.x - vec2.x
    self.y = self.y - vec2.y
    self.z = self.z - vec2.z
    return self
end
function XVECTOR3:MulFloatWith(v)
    self.x = self.x * v
    self.y = self.y * v
    self.z = self.z * v
    return self
end
function XVECTOR3:DevFloatWith(v)
    self.x = self.x / v
    self.y = self.y / v
    self.z = self.z / v
    return self
end

function XVECTOR3.__add(arg1, arg2)

    if math.isVec3(arg2) then
        return XVECTOR3(arg1.x + arg2.x, arg1.y + arg2.y, arg1.z + arg2.z)
    end
    error('Illegal operation: ' .. math.typeName(arg1) .. ' + ' .. math.typeName(arg2))
end
function XVECTOR3.__sub(arg1, arg2)
    if math.isVec3(arg2) then
        return XVECTOR3(arg1.x - arg2.x, arg1.y - arg2.y, arg1.z - arg2.z)
    end
    error('Illegal operation: ' .. math.typeName(arg1) .. ' - ' .. math.typeName(arg2))
end
function XVECTOR3.__mul(arg1, arg2)
    if math.isVec3(arg2) and type(arg1) == 'number' then
        return XVECTOR3(arg1 * arg2.x, arg1 * arg2.y, arg1 * arg2.z)
    elseif math.isVec3(arg1) and type(arg2) == 'number' then
        return XVECTOR3(arg1.x * arg2, arg1.y * arg2, arg1.z * arg2)
    elseif math.isVec3(arg1) and math.isVec3(arg2) then
        return XVECTOR3(arg1.x * arg2.x, arg1.y * arg2.y, arg1.z * arg2.z)
    elseif math.isVec3(arg1) and math.isM4(arg2) then
        return math.mul_m4_vec3(arg2, arg1)
    elseif math.isM4(arg1) and math.isVec3(arg2) then
        return math.mul_m4_vec3(arg1, arg2)
    elseif math.isVec3(arg1) and math.isQuaternion(arg2) then
        return math.mul_quaternion_vec3(arg2, arg1)
    elseif math.isQuaternion(arg1) and math.isVec3(arg2) then
        return math.mul_quaternion_vec3(arg1, arg2)
    end
    error('Illegal operation: ' .. math.typeName(arg1) .. ' * ' .. math.typeName(arg2))
end
function XVECTOR3.__div(arg1, arg2)
    if type(arg2) == 'number' then
        return XVECTOR3(arg1.x / arg2, arg1.y / arg2, arg1.z / arg2)
    elseif math.isVec3(arg2) then
        return XVECTOR3(arg1.x / arg2.x, arg1.y / arg2.y, arg1.z / arg2.z)
    end
    error('Illegal operation: ' .. math.typeName(arg1) .. ' / ' .. math.typeName(arg2))
end
function XVECTOR3.__eq(arg1, arg2)
    if math.isVec3(arg1) and math.isVec3(arg2) then
        return arg1.x == arg2.x and arg1.y == arg2.y and arg1.z == arg2.z
    end
    return false
end
function XVECTOR3:__tostring()
    return 'XVECTOR3(x=' .. self.x .. ', y=' .. self.y .. ', z=' .. self.z .. ')'
end
function XVECTOR3:__index(key)
    if key == 'm' then
        return { self.x, self.y, self.z }
    end
    return XVECTOR3[key]
end
function XVECTOR3:__call(...)
    local vec3 = {}
    local args = { ... }
    if #args == 0 then
        vec3.x = 0
        vec3.y = 0
        vec3.z = 0
    elseif #args == 1 then
        local arg = args[1]
        if type(arg) == 'number' then
            vec3.x = arg
            vec3.y = arg
            vec3.z = arg
        elseif type(arg) == 'table' and math.isVec3(arg) then
            vec3.x = arg.x
            vec3.y = arg.y
            vec3.z = arg.z
        else
            error('XVECTOR3() 错误的参数类型 ' .. math.typeName(arg))
        end
    elseif #args == 3 then
        vec3.x = args[1]
        vec3.y = args[2]
        vec3.z = args[3]
    else
        error('XVECTOR3() 错误的参数个数 ' .. #args)
    end
    setmetatable(vec3, XVECTOR3)
    return vec3
end
